@@ -10,7 +10,10 @@
 #' Users can modify the aggregate level by providing an `aggregate_level`
 #' argument to the function. For example, setting `aggregate_level = 4` when
 #' working with calendar *quarters* produces a payment square by occurrence and
-#' development *year*.
+#' development *year*. \cr \cr
+#' Users will also have the option to decide whether to include the out-of-bound
+#' transactions to the maximum DQ, or to leave them in a separate "tail" cell,
+#' see Details.
 #'
 #' @param frequency_vector a vector of claim frequencies for all the periods.
 #' @param payment_time_list (compound) list of payment times (both the continous
@@ -24,15 +27,21 @@
 #' @param future logical; if true shows the full claim triangle (i.e. including
 #' claim payments in future periods), else shows only the past triangle (default
 #' TRUE).
+#' @param adjust logical; if true accumulates all payments beyond the max
+#' development period to the last development period, else shows a separate
+#' "tail" column for all the out-of-bound transactions.
 #' @return An array of claims payments.
 #' @details
-#' **Remark on out-of-bound payment times**: This function includes adjustment
+#' **Remark on out-of-bound payment times**: This function allows adjustment
 #' for out-of-bound transaction dates, by forcing payments that were projected
 #' to fall out of the maximum development period to be paid at the exact end of
-#' the maximum development period allowed. For example, if we consider 40
-#' periods of development and a claim incurred in the interval (20, 21] was
-#' projected to have a payment at time 62.498210, then we would treat such a
-#' payment as if it occurred at time 60 for the purpose of tabulation.
+#' the maximum development period allowed (when we set `adjust = TRUE`, which is
+#' the default behaviour). For example, if we consider 40 periods of development
+#' and a claim incurred in the interval (20, 21] was projected to have a payment
+#' at time 62.498210, then for the purpose of tabulation, we can
+#' * treat such a payment as if it occurred at time 60 (`adjust = TRUE`);
+#' * leave the payment in the "tail" cell, so the user can see the proportion of
+#' payments beyond the maximum development period (`adjust = FALSE`).
 #' @examples
 #' attach(test_claims_object)
 #' # a square of cumulative claims payments by accident and development quarters
@@ -46,12 +55,24 @@ claim_output <- function(
   payment_size_list,
   aggregate_level = 1,
   incremental = TRUE,
-  future = TRUE) {
+  future = TRUE,
+  adjust = TRUE) {
 
   I <- length(frequency_vector)
-  output_incremental <- array(0, c(I, I))
+  if (adjust == TRUE) {
+    # we will accumulate all out-of-bound transactions to the end of the max DQ
+    output_incremental <- array(0, c(I, I))
+    colnames(output_incremental) <- paste0("DP", 1:I)
+    rownames(output_incremental) <- paste0("AP", 1:I)
+  } else {
+    # need one more column to track all payments beyond the max DQ
+    output_incremental <- array(0, c(I, I + 1))
+    colnames(output_incremental) <- c(paste0("DP", 1:I), "tail")
+    rownames(output_incremental) <- paste0("AP", 1:I)
+  }
   adjustment <- 0 # track the number of corrections required for keeping all the
                   # payments within the bound
+
   # convert to discrete time scale
   payment_period_list <- lapply(payment_time_list, lapply, ceiling)
   for (i in 1:I) {
@@ -60,12 +81,21 @@ claim_output <- function(
       t <- payment_period_list[[i]][[j]] # t is a vector
       no_payment <- length(pmt)
       for (k in 1:no_payment) {
-        # ADJUSTMENT FOR OUT-OF-BOUND TRANSACTION TIMES
         if (t[k] - i + 1 > I) {
-          t[k] <- i + I - 1
+          # out-of-bound transactions
           adjustment <- adjustment + 1
+          if (adjust == TRUE) {
+            # ADJUSTMENT FOR OUT-OF-BOUND TRANSACTION TIMES
+            t[k] <- i + I - 1
+            output_incremental[i, t[k] - i + 1] <- output_incremental[i, t[k] - i + 1] + pmt[k]
+          } else {
+            # accumulate into the tail cell
+            output_incremental[i, "tail"] <- output_incremental[i, "tail"] + pmt[k]
+          }
+        } else {
+          output_incremental[i, t[k] - i + 1] <- output_incremental[i, t[k] - i + 1] + pmt[k]
         }
-        output_incremental[i, t[k] - i + 1] <- output_incremental[i, t[k] - i + 1] + pmt[k]
+
       }
     }
   }
@@ -82,7 +112,17 @@ claim_output <- function(
     # quarterly data at a yearly level (but in a zig-zag fashion)
     new_side_length <- I / aggregate_level
     output_incremental_orig <- output_incremental
-    output_incremental <- array(0, c(new_side_length, new_side_length))
+
+    if (adjust == TRUE) {
+      output_incremental <- array(0, c(new_side_length, new_side_length))
+      colnames(output_incremental) <- paste0("DP", 1:new_side_length)
+      rownames(output_incremental) <- paste0("AP", 1:new_side_length)
+    } else {
+      output_incremental <- array(0, c(new_side_length, new_side_length + 1))
+      colnames(output_incremental) <- c(paste0("DP", 1:new_side_length), "tail")
+      rownames(output_incremental) <- paste0("AP", 1:new_side_length)
+    }
+
     for (i in 1:new_side_length) {
       side_occurrence <- (aggregate_level*(i-1) + 1): (aggregate_level*i)
       for (j in 1:new_side_length) {
@@ -98,6 +138,14 @@ claim_output <- function(
             sum(square_prev[!select])
         }
       }
+
+      if (adjust == TRUE) {
+        # accumulate everything to the last development period
+        output_incremental[i, j] <- output_incremental[i, j] + sum(square_curr[!select])
+      } else {
+        output_incremental[i, "tail"] <-
+          sum(square_curr[!select]) + sum(output_incremental_orig[side_occurrence, "tail"])
+      }
     }
   }
 
@@ -106,11 +154,15 @@ claim_output <- function(
       output_incremental
     } else {
       # only to show the past triangle
+      # note that we won't have yet observed anything beyond the max DQ, so
+      # we simply take the subset output_incremental[, 1:side]
       side <- nrow(output_incremental)
       past_incremental <- array(NA, c(side, side))
-      indicator <- apply(upper.tri(output_incremental, diag = TRUE), 1, rev)
+      colnames(past_incremental) <- paste0("DP", 1:side)
+      rownames(past_incremental) <- paste0("AP", 1:side)
+      indicator <- apply(upper.tri(output_incremental[, 1:side], diag = TRUE), 1, rev)
       for (i in 1:side) {
-        past_incremental[i, 1:sum(indicator[i, ])] <- output_incremental[i, indicator[i, ]]
+        past_incremental[i, 1:sum(indicator[i, ])] <- output_incremental[, 1:side][i, indicator[i, ]]
       }
       past_incremental
     }
@@ -125,9 +177,11 @@ claim_output <- function(
       # only to show the past triangle
       side <- nrow(output_cumulative)
       past_cumulative <- array(NA, c(side, side))
-      indicator <- apply(upper.tri(output_cumulative, diag = TRUE), 1, rev)
+      colnames(past_cumulative) <- paste0("DP", 1:side)
+      rownames(past_cumulative) <- paste0("AP", 1:side)
+      indicator <- apply(upper.tri(output_cumulative[, 1:side], diag = TRUE), 1, rev)
       for (i in 1:side) {
-        past_cumulative[i, 1:sum(indicator[i, ])] <- output_cumulative[i, indicator[i, ]]
+        past_cumulative[i, 1:sum(indicator[i, ])] <- output_cumulative[, 1:side][i, indicator[i, ]]
       }
       past_cumulative
     }
